@@ -1,16 +1,14 @@
-from contextlib import contextmanager
 import importlib
-from pathlib import Path
-from os import PathLike
 import pkgutil
 import signal
 import sys
-from typing import Union, Optional, Set, Iterable
+from contextlib import contextmanager
+from pathlib import Path
+from typing import Iterable, Optional, Set, Union
 
+from .aliases import PathOrStr
 from .exceptions import SigTermReceived
 
-PathOrStr = Union[str, PathLike]
-
 
 def _handle_sigterm(sig, frame):
     raise SigTermReceived
