@@ -1,6 +1,5 @@
+# -*- coding: UTF-8 -*-
 """
-Components for Tango integration with `PyTorch <https://pytorch.org/>`_.
-
 .. important::
     To use this integration you should install ``tango`` with the "torch" extra
     (e.g. ``pip install tango[torch]``) or just install PyTorch after the fact.
@@ -10,6 +9,8 @@ Components for Tango integration with `PyTorch <https://pytorch.org/>`_.
     `pytorch.org/get-started/locally/ <https://pytorch.org/get-started/locally/>`_
     for more details.
 
+Components for Tango integration with `PyTorch <https://pytorch.org/>`_.
+
 These include a basic training loop :class:`~tango.step.Step` and registrable versions
 of many ``torch`` classes, such :class:`torch.optim.Optimizer` and :class:`torch.utils.data.DataLoader`.
 
@@ -100,9 +101,9 @@ would produce the following output:
 
 .. testoutput::
 
-    ● Starting run for "data"
+    ● Starting run for "data"...
     ✓ Finished run for "data"
-    ● Starting run for "train"
+    ● Starting run for "train"...
     Loading best weights from state_worker0_step100.pt
     ✓ Finished run for "train"
 
@@ -140,9 +141,9 @@ __all__ = [
     "StopEarly",
 ]
 
-from .data import DataLoader, Sampler, DataCollator, ConcatTensorDictsCollator
+from .data import ConcatTensorDictsCollator, DataCollator, DataLoader, Sampler
 from .format import TorchFormat
 from .model import Model
-from .optim import Optimizer, LRScheduler
+from .optim import LRScheduler, Optimizer
 from .train import TorchTrainStep
-from .train_callback import TrainCallback, StopEarly
+from .train_callback import StopEarly, TrainCallback
