@@ -1,18 +1,19 @@
 from pathlib import Path
-from typing import Optional, Dict, Any, List
+from typing import Any, Dict, List, Optional
 
 import torch
 
-from .data import DataLoader
-from .model import Model
-from .optim import Optimizer, LRScheduler
 from tango.common.exceptions import TangoError
 from tango.common.registrable import Registrable
 
+from .data import DataLoader
+from .model import Model
+from .optim import LRScheduler, Optimizer
+
 
 class TrainCallback(Registrable):
     """
-    A ``TrainCallback`` is a :class:`~tango.common.registrable.Registrable` class
+    A ``TrainCallback`` is a :class:`~tango.common.Registrable` class
     that can be used within :class:`TorchTrainStep` to customize behavior in the training
     loop.
 
